/*
 *
 * This file is part of the LXARDOSCOPE package.
 *
 * LXARDOSCOPE is an Arduino based oscilloscope for Linux, using the Xforms library.
 *
 * Copyright (C) 2011 Oskar Leuthold
 * 
 * LXARDOSCOPE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LXARDOSCOPE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with LXARDOSCOPE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

// DEFAULT_SAMPLING_RATE is used in file.c and plt.c
#define DEFAULT_SAMPLING_RATE 3000
#define DEFAULT_NBSAMPLES 1024
#define DEFAULT_BAUD_RATE 115200
#define DEFAULT_NUMBER_CHAN    2
#define DEFAULT_BITS_PER_SAMPLE       16
#define DEFAULT_REFERENCE  5.0
#define DEFAULT_TIME_OFFSET 0.0
#define DEFAULT_TIME_SCALE 10.0
#define DEFAULT_TRIG_LEVEL 0
#define DEFAULT_CAL_FACTOR 1.0
#define DEFAULT_TRIG_CHANNEL 0


enum OPERATION_MODE {
			    OPERATION_MODE_NORMAL=0,
			    OPERATION_MODE_ADD=1,
			    OPERATION_MODE_XY=2
			   };
typedef enum{ ASC_TRIG=0, DESC_TRIG=1 } TRIG_TYPE;
typedef enum{ TRIG_OFF=0, TRIG_ON=1 } TRIG_ENB;

struct base_info
{
 float reference;
 float sampling_rate;
 int baud_rate;
 int op_mode;
 int nbsamples; 		// number of samples per chunk 
 int disp_size;			// this is the number of data points to be plotted. Typically is some round value 
						// plus one, due to the data point at zero.
 char devices[12][12]; 	// for serial port names: /dev/ttySX
 int report_timing;
 char device_name[20];
 int dev_h;				// soundcard device handle
 int rchunk;     		// number of chunks read in
 int cchunk;     		// number of chunks copied
 int tchunk;			// number of chunks in tbuf (from 0 to 63)
 int trig;				// trigger index
 int threadenb;			// enable flag for reading thread
 int thread_running;	// indicates that thread is running
 int record;			// flag for writing data to raw file 
 const char *rfname;	// name of output raw file
 int rhandle;			// handle of output raw file
 int ifile;				// flag for reading data from raw file instead of sound card
						// 0: soundcard  1: file open, reading  2: finished, file closed 
 const char *ifname;	// input file name
 int ifsize;			// input file size (samples for each channel)
 int ihandle;			// input file handle
 long int idatapos;	  	// input data position
 int run;				// flag for processing and displaying data
 int tcount;			// number of samples in tbuf (used with file input)
 int tstart;			// first sample to be displayed (with file input)
 float charspersec;		// characters per second (as calculated in lx_run)
 int gndcal;			// flag for GND calibration
 float wx;				// pointer x value
 int pointer_update;			// 1: pointer needs to be updated
};

struct horizontal_info
{
 int trig_chan; 
 float T_pos;         	// time offset
 float T_scale;       	// this is value from GUI counter, full display time
                      	// due to scaling issues, the notion of time per div
						// is much less convenient.
 float trig_level;
 TRIG_TYPE trig_mode;
 TRIG_ENB trig_enb;
};

struct channel_info
{
 int on[2];
 FL_COLOR color[2];
 float V_pos[2];
 float V_scale[2];
 int inv[2];
 float min[2],max[2],pp[2],rms[2],avg[2],offset[2];
};

// for serial port
//struct termios oldtio,newtio;


int timeval_subtract(struct timeval *tresult, struct timeval *tx, struct timeval *ty);
int plot_sound(FL_OBJECT *xyplot);
int plot_file(FL_OBJECT *xyplot, int button);
int open_dev();
//void reset_dev();
//void set_rate(int r);
//void close_dev();
float get_sample(int chan,int i);


