/*
 *
 * This file is part of the LXARDOSCOPE package.
 *
 * LXARDOSCOPE is an Arduino based oscilloscope for Linux, using the Xforms library.
 *
 * Copyright (C) 2011 Oskar Leuthold
 * 
 * LXARDOSCOPE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LXARDOSCOPE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with LXARDOSCOPE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

#include "forms.h"
#include "lxforms.h"
#include "lxa.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <math.h>
#include <sys/dir.h>
#include <pthread.h>
#include <sys/types.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <termios.h>

extern FD_lxa *lxa;
extern FD_about *about;
extern  int alphasort();
int init_lxa_dev();
void set_lxa_time_scale();
void set_sampling_rate();
int xyplot_display_handler(FL_OBJECT *,int,FL_Coord,FL_Coord,int,void *);
double scaler(double val, int button);
void prepare_xyplot(FL_OBJECT *xyplot);
void find_tty_devices();
void select_device(char device_name);

//**********************************

base_info 		BASE={ DEFAULT_REFERENCE,
				DEFAULT_SAMPLING_RATE,
				DEFAULT_BAUD_RATE,
				OPERATION_MODE_NORMAL,
    			DEFAULT_NBSAMPLES, 
				};


horizontal_info	HOR={	DEFAULT_TRIG_CHANNEL,
				DEFAULT_TIME_OFFSET,
				DEFAULT_TIME_SCALE,
				DEFAULT_TRIG_LEVEL,
				ASC_TRIG,
				TRIG_OFF
				}; 
		
			 
channel_info CHAN={{1,1},{FL_RED,FL_GREEN},{2.0,-2.0},{2.0,2.0},{0,0}};


int chunk;
float *dx,*dy1,*dy2,*tbuf[2];
signed short *rbuf,*ibuf;
pthread_t thread;
int  iret;
struct termios oldtio,newtio;
//************************************************************************
/* serial data structure:
   byte 1  A0 5 most significant bits + 224 (128+64+32)
   byte 2  A0 5 least significant bits + 96 (64+32)
   byte 3  A1 5 most significant bits +  96 (64+32)
   byte 4  A1 5 least significant bits +  96 (64+32)
*/



void *lx_run(void *arg)
        {
//		printf("reading thread started\n");
        int res,slot,start;
        unsigned char bufn[256];
		unsigned char bufj[4]={0,0,0,0};
		int n,i,k,j;  
        int nbchar;
		short ch[2];
		int valid;
		float calcount,calval1,calval2;
		int minval[2], maxval[2];	 
        struct timeval tres;
        timeval sta;
        timeval fin;
		nbchar=0;
		slot=BASE.rchunk%8;
		start=slot*BASE.nbsamples*2;
		i=0;
		j=0;
		n=0;
		k=0; // index for rbuf
		calval1=0;
		calval2=0;
		calcount=0;
		minval[0]=0;
		minval[1]=0;
		maxval[0]=0;
		maxval[1]=0;
		int pulse=0;
		if (open_dev()==FALSE) return 0;

		// flushing helps for startup
		tcflush(BASE.dev_h, TCIFLUSH);
		gettimeofday(&sta, NULL);
        res = read(BASE.dev_h,bufn,255);   // returns after 200 chars have been input //
		nbchar=nbchar+res;
		while (BASE.threadenb) {
			if(bufn[n]>223) {
				j=0;
				bufj[0]=bufn[n];
			}
			else if(j==1) bufj[1]=bufn[n];
			else if(j==2) bufj[2]=bufn[n];
			else if(j==3) bufj[3]=bufn[n];
			j++;
			n++;
			if(n>=res) {
				res = read(BASE.dev_h,bufn,255);
				nbchar=nbchar+res;
			n=0;
			}

			if(j==4) { 
				valid=1;
				ch[0]=0;
				ch[1]=0;
				ch[0]=(bufj[0]-224)*32;
				if(bufj[1]>95) {
					ch[0]=ch[0]+bufj[1]-96;
				}
				else {
					printf("error: byte [%d]=%d #2; samples ignored\n",n,bufj[1]);
					valid=0;
				}
				if(bufj[2]>95) {
					ch[1]=(bufj[2]-96)*32;
				}
				else {
					printf("error: byte [%d]=%d #3; samples ignored\n",n,bufj[2]);
					valid=0;
				}
				if(bufj[3]>95) {
					ch[1]=ch[1]+bufj[3]-96;
				}
				else {
					printf("error: byte [%d]=%d #4; samples ignored\n",n,bufj[3]);
					valid=0;
				}
				if(valid) {
					if(ch[0]==0) minval[0]++;
					if(ch[1]==0) minval[1]++;
					if(ch[0]==1023) maxval[0]++;
					if(ch[1]==1023) maxval[1]++;
					rbuf[start+k]=(short)(ch[0]-CHAN.offset[0]);
					rbuf[start+k+1]=(short)(ch[1]-CHAN.offset[1]);
					if(BASE.gndcal) {
						calval1=calval1+(float)ch[0];
						calval2=calval2+(float)ch[1];
						calcount++;
					}
					k=k+2;

				if(BASE.record==1) {
					int e;
   					if((e=write(BASE.rhandle,&ch,4))<0) {
      					fprintf(stderr, "write failed %s\n", strerror(errno));
      					close(BASE.rhandle);
      					exit(1);
    				}
				}

// tick mark for display
//				if(k>2*BASE.nbsamples-5) {
//					rbuf[start+2*BASE.nbsamples-4]=(short)(-100);				
//					rbuf[start+2*BASE.nbsamples-3]=(short)(-100);		
//					rbuf[start+2*BASE.nbsamples-2]=(short)(-100);		
//					rbuf[start+2*BASE.nbsamples-1]=(short)(-100);
				if(k>=2*BASE.nbsamples) {
					k=0;
					BASE.rchunk++;
					slot=BASE.rchunk%8;
					start=slot*BASE.nbsamples*2;

  					if(minval[0]>0) fl_set_object_lcolor(lxa->text_min_chan1,FL_RED);
 	 				else fl_set_object_lcolor(lxa->text_min_chan1,FL_BLACK);
 	 				if(maxval[0]>0) fl_set_object_lcolor(lxa->text_max_chan1,FL_RED);
 	 				else fl_set_object_lcolor(lxa->text_max_chan1,FL_BLACK);
  					if(minval[1]>0) fl_set_object_lcolor(lxa->text_min_chan2,FL_RED);
  					else fl_set_object_lcolor(lxa->text_min_chan2,FL_BLACK);
 	 				if(maxval[1]>0) fl_set_object_lcolor(lxa->text_max_chan2,FL_RED);
  					else fl_set_object_lcolor(lxa->text_max_chan2,FL_BLACK);
					minval[0]=0; minval[1]=0; maxval[0]=0; maxval[1]=0;

					if(BASE.rchunk%10==9) {
						gettimeofday(&fin, NULL);
    					timeval_subtract(&tres,&fin,&sta);
    					double tspan=1e-6*(double)(1000000*tres.tv_sec+tres.tv_usec);
// update the real rate:
						BASE.charspersec=nbchar/tspan;
						BASE.sampling_rate=BASE.charspersec/4.0;
						set_lxa_time_scale();
						if(BASE.sampling_rate<BASE.baud_rate/50) {
							printf("serial port communication too slow\n");
							printf("  receiving %d characters per second\n",(int)BASE.charspersec);
							printf("  expecting at least %d characters per second\n",
										(int)(BASE.baud_rate/12.5));
						}
						if(BASE.report_timing==1) {
printf("***** elapsed time=%2.6f, %d bytes received, %d bytes per second, sampling rate= %d\n",
							tspan,nbchar,(int)(BASE.charspersec),(int)BASE.sampling_rate);
							}
						if(BASE.gndcal>0) {
							CHAN.offset[0]=calval1/calcount;
							CHAN.offset[1]=calval2/calcount;
							printf("          offset for channel A0 = %d, for A1 = %d\n",
								(int)CHAN.offset[0],(int)CHAN.offset[1]); 
							BASE.gndcal++;
							if(BASE.gndcal>3) {
								BASE.gndcal=0;
								fl_set_object_color(lxa->button_gndcal,FL_MCOL,FL_DARKORANGE);
								calval1=0;
								calval2=0;
								calcount=0;
							}
						} //if(BASE.gndcal>0)
					}//if(BASE.rchunk%10==9)
					if(BASE.rchunk%3==0) {
						if(pulse==0) {
							pulse=1;
							fl_set_object_color(lxa->button_pulse,FL_PALEGREEN,FL_ORCHID);
						}
						else {
							pulse=0;
							fl_set_object_color(lxa->button_pulse,FL_ORCHID,FL_PALEGREEN);
						}
					}
			    }//	if(k>2*BASE.nbsamples-5)	
			} //if(valid)
		} // if(j>3)
	} //while (BASE.threadenb)


tcsetattr(BASE.dev_h,TCSANOW,&oldtio);

BASE.thread_running=0;	

return 0;
}





int init_lxa(char *device_name)
 {  
   const char *undef="undefined";
   const char *tfile="file";
   int select;
   find_tty_devices();
   fl_clear_choice(lxa->droplist_dev);
   strcpy(BASE.device_name,device_name);
   if(strcmp(device_name,undef)==0) {     // no external input
   		fl_addto_choice(lxa->droplist_dev,undef);
   		fl_set_object_color(lxa->droplist_dev,FL_DARKORANGE,FL_INACTIVE);
   }
   else {
        if (init_lxa_dev()==0) {
   	        printf("can not initialize device %s\n",device_name);
            select=fl_addto_choice(lxa->droplist_dev,undef);
            fl_set_object_color(lxa->droplist_dev,FL_DARKORANGE,FL_INACTIVE);
        }
        else {
            fl_addto_choice(lxa->droplist_dev,device_name);
            fl_set_object_color(lxa->droplist_dev,FL_MCOL,FL_INACTIVE);
            fl_set_button(lxa->button_run,1);
     		fl_set_timer(lxa->timer_refresh,0.01);
			fl_activate_object(lxa->button_run);
			fl_show_object(lxa->button_run);
     		fl_set_object_lcol(lxa->button_run,FL_RED);
     		fl_set_object_label(lxa->button_run,"STOP");
	 		BASE.rchunk=0;
     		BASE.cchunk=0;
	 		BASE.threadenb=1;
			BASE.run=1;
     		fl_deactivate_object(lxa->record);	
        }
   }
   int nbdev=atoi(BASE.devices[0]);
   for(int n=1;n<=nbdev;n++) {
         if(strcmp(device_name,BASE.devices[n])!=0)
         fl_addto_choice(lxa->droplist_dev,BASE.devices[n]);  
		 } 
   fl_addto_choice(lxa->droplist_dev,tfile); 
// reserve buffers for waveform data
    int n=6*DEFAULT_SAMPLING_RATE+1;            // for 5 seconds of data, plus some extra in case ....
    dx=new float[n];
    dy1=new float[n];
    dy2=new float[n];
    tbuf[0]=new float[BASE.nbsamples*64];   	//temp buffer for channel 0
    tbuf[1]=new float[BASE.nbsamples*64];   	//temp buffer for channel 1
    rbuf= new signed short[BASE.nbsamples*16];	//read-in buffer
	ibuf= new signed short[BASE.nbsamples*2];	//input buffer
   fl_set_object_posthandler(lxa->xyplot_display,xyplot_display_handler);
   fl_set_xyplot_inspect(lxa->xyplot_display,1);
   fl_set_counter_return(lxa->counter_Vpos1,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->counter_Vscale1,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->counter_Vpos2,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->counter_Vscale2,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->counter_Tpos,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->counter_Tscale,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->thumb_trig_level,FL_RETURN_CHANGED);
   fl_set_counter_return(lxa->thumb_reference,FL_RETURN_CHANGED);
//   set_sampling_rate();  // lookup sampling rate from baud rate
   set_lxa_time_scale();
   printf("** initial sampling rate set to %d\n",(int)BASE.sampling_rate);
   BASE.trig=0;
   BASE.record=0;
   BASE.ifile=0;
   BASE.report_timing=0;  
   BASE.thread_running=0; 
	BASE.pointer_update=0;
   return 1;
 }


void find_tty_devices()
{
        int i=1;
		int done=0;
        DIR *pDIR;
        struct dirent *entry;
        pDIR=opendir("/dev");
        if(pDIR!=NULL) {
                while (done==0) {
					entry = readdir(pDIR);
					if(entry!=NULL) {
               			if (strncmp(entry->d_name, "ttyS", 4)== 0) {
                    		strcpy(BASE.devices[i],"/dev/");
							strcat(BASE.devices[i],entry->d_name);
							i++;
							if(i>9) done=1;
                		}
                	}
					else done=1;
                }
                closedir(pDIR);
                sprintf(BASE.devices[0],"%d",i-1);
        }
}

int init_lxa_dev()
 { 
//printf("init_lxa_dev\n");
   const char *undef="undefined";
   if (strcmp(BASE.device_name,undef)==0) return 0;
   set_sampling_rate();
//   if (open_dev()==FALSE) return 0;
//   reset_dev();
//   set_rate(BASE.sampling_rate);
   return 1;
 }


//void reset_lxa_dev()  {
//   close_dev();
//   sleep(1);
//   init_lxa_dev();
// }

void set_lxa_time_scale() {
   BASE.disp_size=HOR.T_scale*BASE.sampling_rate/1000+1;   
   fl_set_scaler_value(lxa->counter_Tscale,HOR.T_scale);
//printf("BASE.disp_size=%d\n",BASE.disp_size);
//printf("HOR.T_scale=%f\n",HOR.T_scale);
//printf("BASE.sampling_rate=%f\n",BASE.sampling_rate);
}


// the following relationship is arbitrary ...
// used only at startup. Value adjusted after first chunk.
void set_sampling_rate() {
	switch (BASE.baud_rate)
      {
         case 115200:
            BASE.sampling_rate = 3000;
            break;
         case 38400:
            BASE.sampling_rate = 1000;
            break;
         case 19200:
            BASE.sampling_rate  = 500;
            break;
         case 9600:
            BASE.sampling_rate  = 300;
            break;
         case 4800:
            BASE.sampling_rate  = 200;
            break;
         case 2400:
            BASE.sampling_rate  = 100;
            break;
         case 1800:
            BASE.sampling_rate  = 80;
            break;
         case 1200:
            BASE.sampling_rate  = 50;
            break;
         case 600:
            BASE.sampling_rate  = 30;
            break;
         case 300:
            BASE.sampling_rate  = 20;
            break;
		}
printf("**  sampling rate initialized to %d\n",(int)BASE.sampling_rate);
}




void refresh_mouse_info(float wx, float wy)  {
char cad[16];
BASE.wx=wx; 
BASE.pointer_update=1;
switch (BASE.op_mode) {
   	case OPERATION_MODE_NORMAL: {
  		if(HOR.T_scale<10.0) sprintf(cad,"%.3f",wx);
  		else if (HOR.T_scale<100.0) sprintf(cad,"%.2f",wx);
  		else if (HOR.T_scale<1000.0) sprintf(cad,"%.1f",wx);
  		else sprintf(cad,"%.0f",wx);
  		fl_set_object_label(lxa->text_mouse_T,cad);

		fl_set_object_label(lxa->text_mouse_V1,"*  ");
		fl_set_object_label(lxa->text_mouse_V2,"*  ");

		break;
		}
	case OPERATION_MODE_ADD: {
  		if(HOR.T_scale<10.0) sprintf(cad,"%.3f",wx);
  		else if (HOR.T_scale<100.0) sprintf(cad,"%.2f",wx);
  		else if (HOR.T_scale<1000.0) sprintf(cad,"%.1f",wx);
  		else sprintf(cad,"%.0f",wx);
  		fl_set_object_label(lxa->text_mouse_T,cad);

		fl_set_object_label(lxa->text_mouse_V1,"*  ");
		fl_set_object_label(lxa->text_mouse_V2,"*  ");

		break;
	}
	case OPERATION_MODE_XY: {

		fl_set_object_label(lxa->text_mouse_V1,"*  ");
		fl_set_object_label(lxa->text_mouse_V2,"*  ");
		fl_set_object_label(lxa->text_mouse_T,"*  ");
		
		break;
	}
  }
  if(BASE.ifile) plot_file(lxa->xyplot_display,3);  
}


/* callbacks and freeobj handles for form lxa */

// the following works only if mouse click are near line with x=0 or line with y=0.
void CB_xyplot_display(FL_OBJECT *ob, long data)
{
  float x,y;
  int i;
  fl_get_xyplot(ob,&x,&y,&i);
  printf("MOUSE CLICK ==> T=%.4f, V=%.4f\n",x,y);
}

void CB_logotipo(FL_OBJECT *ob, long data)
{
   fl_show_form(about->about,FL_PLACE_CENTERFREE,FL_FULLBORDER,"About...");
}

void CB_button_run(FL_OBJECT *ob, long data)
{
  int b=fl_get_button(ob);   
  const char *undef="undefined";
  if(strcmp(BASE.device_name,undef)==0) {
  	printf("Please select a valid device, then press START\n"); 
  	BASE.run=0;
  	fl_set_button(lxa->button_run,0);
//  	fl_set_timer(lxa->timer_refresh,0.0);
  	fl_set_object_lcol(ob,FL_GREEN);
  	fl_set_object_label(ob,"START");
  }
  else if (b==1) {
     BASE.run=1;
     fl_set_timer(lxa->timer_refresh,0.01);
     fl_set_object_lcol(ob,FL_RED);
     fl_set_object_label(ob,"STOP");
	 BASE.rchunk=0;
     BASE.cchunk=0;
	 BASE.tchunk=0;
	 BASE.threadenb=1;
     fl_deactivate_object(lxa->record);	
     fl_deactivate_object(lxa->droplist_dev);
	 fl_deactivate_object(lxa->droplist_rate);
	 iret=pthread_create( &thread, NULL, *lx_run,NULL);
	 if(iret==1)
     		printf("Thread creation failed: %d\n", iret);
	 else BASE.thread_running=1;
  }
  else  {
     BASE.run=0;
     fl_set_object_lcol(ob,FL_GREEN);
     fl_set_object_label(ob,"START");
	 fl_set_timer(lxa->timer_refresh,0.0);
	 BASE.threadenb=0;
	 fl_activate_object(lxa->record);
     fl_activate_object(lxa->droplist_dev);
	 fl_activate_object(lxa->droplist_rate);
   }
}


void CB_timer_refresh(FL_OBJECT *ob, long data)
{
    switch (HOR.trig_enb)
    {
     case TRIG_OFF:       
         fl_set_object_color(lxa->indicator_triggered,FL_INACTIVE,FL_INACTIVE);
      break;
     case TRIG_ON: 
          fl_set_object_color(lxa->indicator_triggered,FL_RED,FL_RED);
     break;
    }
  if(BASE.thread_running==0) {
	iret = pthread_create( &thread, NULL, *lx_run,NULL);
	if(iret==1) {
     	printf("Thread creation failed: %d\n", iret);
		return;
	}
	else {
		BASE.thread_running=1;
	}
  }
//  refresh_lxa();
  if (BASE.run==1) { 
 	fl_freeze_form(lxa->lxa);
 	plot_sound(lxa->xyplot_display);
 	fl_unfreeze_form(lxa->lxa);
    fl_set_timer(lxa->timer_refresh,0.3);
  }
  else fl_set_timer(lxa->timer_refresh,0.0);
}


int xyplot_display_handler(FL_OBJECT *ob,int event,FL_Coord mx,FL_Coord my,int key,void *xev)
{
 //static int p=1;

switch(event)
  {
    case FL_MOUSE:

                float wx,wy;
                fl_xyplot_s2w(ob,mx,my,&wx,&wy);
                refresh_mouse_info(wx,wy);
                 break;
/*
    case FL_PUSH:
		if (fl_mouse_button()==1) p=1;
		else p=-1;
		break;
*/

  }

 return 0;

}


// adapted from http://www.opensource.apple.com/source/distcc/distcc-20/src/timeval.c
// modified to leave structures tx and ty intact 
int timeval_subtract (struct timeval *tresult,
                      struct timeval *tx,
                      struct timeval *ty)
     { 
       long int ty_tv_usec=ty->tv_usec;
       long int ty_tv_sec=ty->tv_sec;
       //Perform the carry for the later subtraction by updating y.
       if (tx->tv_usec < ty->tv_usec) {
         int nsec = (ty->tv_usec - tx->tv_usec) / 1000000 + 1;
          ty_tv_usec = ty->tv_usec - 1000000 * nsec;
         ty_tv_sec = ty->tv_sec + nsec;
       }
       if (tx->tv_usec - ty->tv_usec > 1000000) {
         int nsec = (tx->tv_usec - ty->tv_usec) / 1000000;
         ty_tv_usec = ty->tv_usec + 1000000 * nsec;
         ty_tv_sec = ty->tv_sec - nsec;
       }
     
       // Compute the time remaining to wait. tv_usec is certainly positive. 
       tresult->tv_sec = tx->tv_sec - ty_tv_sec;
       tresult->tv_usec = tx->tv_usec - ty_tv_usec;
     
       // Return 1 if result is negative. 
       return tx->tv_sec < ty->tv_sec;
     }


//**********************************

void CB_counter_Vpos1(FL_OBJECT *ob, long)
{
  CHAN.V_pos[0]=fl_get_counter_value(ob);
  if(BASE.ifile) plot_file(lxa->xyplot_display,3);  
}

void CB_counter_Vscale1(FL_OBJECT *ob, long)
{
  int digit;
  double val=fl_get_scaler_value(ob);
  double newval=val;
  int button=fl_get_scaler_button(ob);
  newval=scaler(val,button);
  if(newval<0.002) newval=0.002;
  if(newval>10) newval=10;
  fl_set_scaler_value(ob,newval);
  digit=4;
  if(newval>0.0999) digit=3;
  if(newval>0.9999) digit=2;
  if(newval>9.9999) digit=1;
  CHAN.V_scale[0]=newval;
  fl_set_scaler_precision(ob,digit);
  if(BASE.ifile) plot_file(lxa->xyplot_display,3);  
}

void CB_check_chan1(FL_OBJECT *ob, long)
{
  CHAN.on[0]=fl_get_button(ob);
}

void CB_button_color_chan1(FL_OBJECT *ob, long)
{
  CHAN.color[0]=fl_show_colormap(CHAN.color[0]);
  fl_set_object_color(ob,CHAN.color[0],FL_COL1);
}

void CB_button_inv_chan1(FL_OBJECT *ob, long)
{
  CHAN.inv[0]=fl_get_button(ob);
}


void CB_counter_Vpos2(FL_OBJECT *ob, long)
{
  CHAN.V_pos[1]=fl_get_counter_value(ob);
  if(BASE.ifile) plot_file(lxa->xyplot_display,3);  
}

void CB_counter_Vscale2(FL_OBJECT *ob, long)
{
  int digit;
  double val=fl_get_scaler_value(ob);
  double newval=val;
  int button=fl_get_scaler_button(ob);
  newval=scaler(val,button);
  if(newval<0.002) newval=0.002;
  if(newval>10) newval=10;
  fl_set_scaler_value(ob,newval);
  digit=4;
  if(newval>0.0999) digit=3;
  if(newval>0.9999) digit=2;
  if(newval>9.9999) digit=1;
  CHAN.V_scale[1]=newval;
  fl_set_scaler_precision(ob,digit);
  if(BASE.ifile) plot_file(lxa->xyplot_display,3);  
}


void CB_check_chan2(FL_OBJECT *ob, long)
{
  CHAN.on[1]=fl_get_button(ob);
}

void CB_button_color_chan2(FL_OBJECT *ob, long)
{
  CHAN.color[1]=fl_show_colormap(CHAN.color[1]);
  fl_set_object_color(ob,CHAN.color[1],FL_COL1);
}

void CB_button_inv_chan2(FL_OBJECT *ob, long)
{
  CHAN.inv[1]=fl_get_button(ob);
}

// the following function creates a scale with step sizes
// which are a percentage of the actual value, and allows for
// large and small changes, such that the selectable values
// always make sense in the decimal system.
double scaler(double val, int button) {
  double newval=val;
// printf("val =%e button=%d\n",val,button);
  switch (button) {
  case -2:
	if(val>0.002)	newval=0.002;
	if(val>0.005)	newval=0.005;
	if(val>0.01)	newval=0.01;
	if(val>0.02)	newval=0.02;
	if(val>0.05)	newval=0.05;
	if(val>0.1)	newval=0.1;
	if(val>0.2)	newval=0.2;
	if(val>0.5)	newval=0.5;
	if(val>1)	newval=1;
	if(val>2)	newval=2;
	if(val>5)	newval=5;
      	if(val>10)	newval=10;
      	if(val>20)	newval=20;
      	if(val>50)	newval=50;
      	if(val>100) 	newval=100;
      	if(val>200) 	newval=200;
      	if(val>500) 	newval=500;
      	if(val>1000) 	newval=1000;
      	if(val>2000) 	newval=2000;
	break;
  case -1:
	if(val>0.002)	newval=val-0.0002;
	if(val>0.005)	newval=val-0.0005;
	if(val>0.01)	newval=val-0.001;
	if(val>0.02)	newval=val-0.002;
	if(val>0.05)	newval=val-0.005;
	if(val>0.1)	newval=val-0.01;
	if(val>0.2)	newval=val-0.02;
	if(val>0.5)	newval=val-0.05;
	if(val>1)	newval=val-0.1;
	if(val>2)	newval=val-0.2;
	if(val>5)	newval=val-0.5;
      	if(val>10)	newval=val-1;
      	if(val>20)	newval=val-2;
      	if(val>50)	newval=val-5;
      	if(val>100) 	newval=val-10;
      	if(val>200) 	newval=val-20;
      	if(val>500) 	newval=val-50;
      	if(val>1000) 	newval=val-100;
      	if(val>2000) 	newval=val-200;
	break; 	
  case 1:
	if(val<5000)	newval=val+200;
	if(val<2000)	newval=val+100;
	if(val<1000)	newval=val+50;
	if(val<500)	newval=val+20;
	if(val<200)	newval=val+10;
	if(val<100)	newval=val+5;
	if(val<50)	newval=val+2;
	if(val<20)	newval=val+1;
	if(val<10)	newval=val+0.5;
	if(val<5)	newval=val+0.2;
	if(val<2)	newval=val+0.1;
	if(val<1)	newval=val+0.05;
	if(val<0.5)	newval=val+0.02;
	if(val<0.2)	newval=val+0.01;
	if(val<0.1)	newval=val+0.005;
	if(val<0.05)	newval=val+0.0002;
	if(val<0.02)	newval=val+0.001;
	if(val<0.01)	newval=val+0.0005;
	if(val<0.005)	newval=val+0.0002;
	break;
  case 2:
	if(val<5000)	newval=5000;
	if(val<2000)	newval=2000;
	if(val<1000)	newval=1000;
	if(val<500)	newval=500;
	if(val<200)	newval=200;
	if(val<100)	newval=100;
	if(val<50)	newval=50;
	if(val<20)	newval=20;
	if(val<10)	newval=10;
	if(val<5)	newval=5;
	if(val<2)	newval=2;
	if(val<1)	newval=1;
	if(val<0.5)	newval=0.5;
	if(val<0.2)	newval=0.2;
	if(val<0.1)	newval=0.1;
	if(val<0.05)	newval=0.05;	
	if(val<0.02)	newval=0.02;
	if(val<0.01)	newval=0.01;
	if(val<0.005)	newval=0.005;	
	break;
}
return newval;
}


void CB_counter_Tscale(FL_OBJECT *ob, long)
{
  int digit;
  double srate;
  double val=fl_get_scaler_value(ob);
  double newval=val;
  int button=fl_get_scaler_button(ob);
  newval=scaler(val,button);

  if(newval<0.1) newval=0.1;
  if(newval>5000) newval=5000;
  fl_set_scaler_value(ob,newval);

digit=3;
if(newval>0.9999) digit=2;
if(newval>9.9999) digit=1;
if(newval>99.999) digit=0;

  fl_set_scaler_precision(ob,digit);
  srate=BASE.sampling_rate;
  BASE.disp_size=newval*srate/1000+1; 
  HOR.T_scale=newval;
  BASE.trig=0; 
  if(BASE.ifile) plot_file(lxa->xyplot_display,3);    
}

void CB_counter_Tpos(FL_OBJECT *ob, long)
{
  HOR.T_pos=fl_get_counter_value(ob);
//printf("%.1f\n",HOR.T_pos);
}


void CB_thumb_trig_level(FL_OBJECT *ob, long)
{
  double val=fl_get_thumbdig_value(ob);
  HOR.trig_level=val;
}


void CB_droplist_rate(FL_OBJECT *ob, long) {
  float r;
  sscanf(fl_get_choice_text(ob),"%f",&r);
  BASE.baud_rate=(int)r;
  set_sampling_rate();
//  reset_lxa_dev();
   sleep(1);
   init_lxa_dev();
   set_lxa_time_scale();
}

void CB_droplist_dev(FL_OBJECT *ob, long) {
  char test[20];
  char undef[]="undefined";
  char tfile[]="file";
  char nfile[]="(null)";
  const char *fname;
  int success=0;
  strcpy(test,fl_get_choice_text(ob));
  if(strcmp(test,tfile)==0) {		// user wants to display file content
     fl_set_fselector_placement(FL_PLACE_FREE); 
     fl_disable_fselector_cache(1);
	 fname=fl_show_fselector("Select Rawfile",  ".", "*.raw",0);
	 if(fname==NULL) success=0;
     else if (strlen(fname)==0) success=0;
	 else if (strcmp(nfile,fname)==0) success=0;
     else {
	 if((BASE.ihandle=open(fname,O_RDONLY))==-1) {  //open file for display
			printf("Cannot open %s for display\n",BASE.ifname);
			BASE.ifile=0;
        	success=0;
        }
		else success=1;
	 }
	 if(success) {
		BASE.ifname=fname;	
		struct stat filestatus;
		stat(BASE.ifname , &filestatus );
		BASE.ifsize=(int)filestatus.st_size/4;
		printf("file name= >%s<\n",BASE.ifname);
		int fsize=(int)filestatus.st_size;
		if(fsize/(4*BASE.sampling_rate)<10000) 
		printf("file size is %d bytes, (%d samples for each channel, %.2f seconds)\n",
			fsize,BASE.ifsize,fsize/(4*BASE.sampling_rate));	
		else
		printf("file size is %d bytes, (%d samples for each channel, %.3e seconds)\n",
			fsize,BASE.ifsize,fsize/(4*BASE.sampling_rate));					
	    strcpy(BASE.device_name,tfile);
	    BASE.ifile=1;
		fl_show_object(lxa->counter_Pos);
 		fl_hide_object(lxa->button_run);
		BASE.run=0;
  		fl_set_button(lxa->button_run,0);
	  	fl_set_object_lcol(lxa->button_run,FL_GREEN);
  		fl_set_object_label(lxa->button_run,"START");
 		fl_hide_object(lxa->button_run);		
		fl_set_object_color(lxa->record,FL_INACTIVE,FL_COL1);
		fl_deactivate_object(lxa->record);
		if(BASE.record) close(BASE.rhandle);
   		BASE.record=0;
// the first one does not seem to work properly:
		fl_deactivate_object(lxa->check_trigger);
		fl_set_object_color(lxa->check_trigger,FL_LIGHTER_COL1,FL_LIGHTER_COL1);
		fl_set_object_lcol(lxa->check_trigger,FL_DARKER_COL1);
		fl_deactivate_object(lxa->thumb_trig_level);
		fl_set_object_color(lxa->thumb_trig_level,FL_LIGHTER_COL1,FL_LIGHTER_COL1);
		fl_set_object_lcol(lxa->thumb_trig_level,FL_DARKER_COL1);
		fl_deactivate_object(lxa->trig_channel);
		fl_set_object_color(lxa->trig_channel,FL_LIGHTER_COL1,FL_LIGHTER_COL1);
		fl_set_object_lcol(lxa->trig_channel,FL_DARKER_COL1);
		fl_deactivate_object(lxa->trig_mode);
		fl_set_object_color(lxa->trig_mode,FL_LIGHTER_COL1,FL_LIGHTER_COL1);
		fl_set_object_lcol(lxa->trig_mode,FL_DARKER_COL1);
		fl_deactivate_object(lxa->counter_Tpos);
		fl_set_object_color(lxa->counter_Tpos,FL_LIGHTER_COL1,FL_LIGHTER_COL1);
		fl_set_object_lcol(lxa->counter_Tpos,FL_DARKER_COL1);
   		plot_file(lxa->xyplot_display,0);
printf("called plot_file\n");
		}

	else {
		fl_set_choice_text(ob,undef);
		printf("please select a file, or a sound card\n");
	}

   }
   else if(strcmp(test,undef)!=0) {   // one of the dsp entries was selected
       strcpy(BASE.device_name,test);
//       close_dev();
	   if (strcmp(undef,fl_get_choice_item_text(ob,1))==0) {
           if (init_lxa_dev()==0) {
                printf("can not initialize device < %s >, please select a different one\n",test); 
                fl_set_object_color(lxa->droplist_dev,FL_DARKORANGE,FL_INACTIVE); 			
           }
           else {
           		fl_delete_choice(ob,1);
		   		success=1;
 
          		fl_set_object_color(lxa->droplist_dev,FL_MCOL,FL_INACTIVE);
		   		BASE.ifile=0;
 		   		fl_show_object(lxa->button_run);
		   		fl_activate_object(lxa->button_run);
 		   		fl_hide_object(lxa->counter_Pos);	
           }
       }
       else {
          if (init_lxa_dev()==0) {
                printf("can not initialize device < %s >, please select a different one\n",test);
				fl_set_object_color(lxa->droplist_dev,FL_DARKORANGE,FL_INACTIVE); 	
          }
          else success=1;
		}
	    if(success) {
	        fl_set_object_color(lxa->droplist_dev,FL_MCOL,FL_INACTIVE);
		  	BASE.ifile=0; 
		  	fl_show_object(lxa->button_run);
		  	fl_activate_object(lxa->button_run);
 		  	fl_hide_object(lxa->counter_Pos);	
			fl_activate_object(lxa->check_trigger);
			fl_set_object_color(lxa->check_trigger,FL_MCOL,FL_YELLOW);
			fl_set_object_lcol(lxa->check_trigger,FL_BLACK);
			fl_activate_object(lxa->thumb_trig_level);
			fl_set_object_color(lxa->thumb_trig_level,FL_LIGHTER_COL1,FL_INACTIVE);
			fl_set_object_lcol(lxa->thumb_trig_level,FL_BLACK);
			fl_activate_object(lxa->trig_channel);
			fl_set_object_color(lxa->trig_channel,FL_MCOL,FL_MCOL);
			fl_set_object_lcol(lxa->trig_channel,FL_BLACK);
			fl_activate_object(lxa->trig_mode);
			fl_set_object_color(lxa->trig_mode,FL_MCOL,FL_MCOL);
			fl_set_object_lcol(lxa->trig_mode,FL_BLACK);
			fl_activate_object(lxa->counter_Tpos);
			fl_set_object_color(lxa->counter_Tpos,FL_LIGHTER_COL1,FL_INACTIVE);
			fl_set_object_lcol(lxa->counter_Tpos,FL_BLACK);
		}
   }
   else printf("please make a valid choice\n");
}

void CB_record(FL_OBJECT *ob, long) {
	int rec=fl_get_button(ob);
	if(rec==1) {
   		fl_set_fselector_placement(FL_PLACE_FREE); 
		fl_disable_fselector_cache(1);
   		BASE.rfname=fl_show_fselector("Save raw data as:",  0, "*.raw",0);
   		printf("file name=%s\n",BASE.rfname);
   		int fr;
   		if((fr=open(BASE.rfname,O_RDWR|O_CREAT|O_TRUNC,S_IRWXU))==-1)
      		{
         	printf("Cannot open %s\n",BASE.rfname);
         	return;
      	}
    	fl_set_object_color(lxa->record,FL_GREEN,FL_GREEN);
   		BASE.record=1;
		BASE.rhandle=fr;
     }
     else {
		close(BASE.rhandle);
		fl_set_object_color(lxa->record,FL_INACTIVE,FL_COL1);
   		BASE.record=0;
	 }
}

void CB_droplist_opmode(FL_OBJECT *ob, long) {
  	BASE.op_mode=fl_get_choice(ob)-1;
	switch (BASE.op_mode) {
   		case OPERATION_MODE_NORMAL: {
		fl_activate_object(lxa->counter_Vpos2);
		fl_set_object_color(lxa->counter_Vpos2,FL_MCOL,FL_INACTIVE);
		fl_set_object_lcol(lxa->counter_Vpos2,FL_BLACK);
		fl_activate_object(lxa->counter_Vscale2);
		fl_set_object_color(lxa->counter_Vscale2,FL_MCOL,FL_INACTIVE);		
		fl_set_object_lcol(lxa->counter_Vscale2,FL_BLACK);
		break;
		}
		case OPERATION_MODE_ADD: {
		fl_deactivate_object(lxa->counter_Vpos2);
		fl_set_object_color(lxa->counter_Vpos2,FL_LIGHTER_COL1,FL_LIGHTER_COL1);
		fl_set_object_lcol(lxa->counter_Vpos2,FL_DARKER_COL1);
		fl_deactivate_object(lxa->counter_Vscale2);
		fl_set_object_color(lxa->counter_Vscale2,FL_LIGHTER_COL1,FL_LIGHTER_COL1);		
		fl_set_object_lcol(lxa->counter_Vscale2,FL_DARKER_COL1);
		break;
		}
		case OPERATION_MODE_XY: {
		fl_activate_object(lxa->counter_Vpos2);
		fl_set_object_color(lxa->counter_Vpos2,FL_MCOL,FL_INACTIVE);
		fl_set_object_lcol(lxa->counter_Vpos2,FL_BLACK);
		fl_activate_object(lxa->counter_Vscale2);
		fl_set_object_color(lxa->counter_Vscale2,FL_MCOL,FL_INACTIVE);		
		fl_set_object_lcol(lxa->counter_Vscale2,FL_BLACK);
		break;
		}
	}

    fl_set_xyplot_xtics(lxa->xyplot_display, 5, 2);
    fl_set_xyplot_xgrid(lxa->xyplot_display, FL_GRID_MINOR);
}

void CB_check_trigger(FL_OBJECT *ob, long) { 
  HOR.trig_enb=(TRIG_ENB)(fl_get_button(ob));
}

void CB_trig_mode(FL_OBJECT *ob, long) { 
     if((TRIG_TYPE)HOR.trig_mode==1) {
     HOR.trig_mode=(TRIG_TYPE)0;
     fl_set_object_label(lxa->trig_mode,"UP");
     }
     else {
     HOR.trig_mode=(TRIG_TYPE)1;
     fl_set_object_label(lxa->trig_mode, "DN");
     }
}

void CB_trig_channel(FL_OBJECT *ob, long)  { 
     if(HOR.trig_chan) {
     HOR.trig_chan=0;
     fl_set_object_label(lxa->trig_channel,"1");
     }
     else {
     HOR.trig_chan=1;
     fl_set_object_label(lxa->trig_channel, "2");
     }
  }

void CB_thumb_reference(FL_OBJECT *ob, long)  {
  double val=fl_get_thumbdig_value(ob);
  BASE.reference=val;
}

void CB_report_timing(FL_OBJECT *ob, long)  { 
  BASE.report_timing=fl_get_button(ob);
}

void CB_button_about_OK(FL_OBJECT *ob, long) {
  fl_hide_form(about->about);
}

void CB_position(FL_OBJECT *ob, long) {
	  int button=fl_get_scaler_button(ob);
 	  plot_file(lxa->xyplot_display,button);     
}

int CB_at_close(FL_FORM *lxa,void *)  { 
if(BASE.record) close(BASE.rhandle);
if(BASE.ifile)  close(BASE.ihandle);
return FL_OK;
}

void CB_button_gndcal(FL_OBJECT *ob, long)  {
	BASE.gndcal=1;
	fl_set_object_color(lxa->button_gndcal,FL_RED,FL_MCOL);
}

void CB_button_pulse(FL_OBJECT *ob, long)  {
printf("this button is for output only\n");
}

