/*
 *
 * This file is part of the LXARDOSCOPE package.
 *
 * LXARDOSCOPE is an Arduino based oscilloscope for Linux, using the Xforms library.
 *
 * Copyright (C) 2011 Oskar Leuthold
 * 
 * LXARDOSCOPE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LXARDOSCOPE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with LXARDOSCOPE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

#include "forms.h"
#include "lxforms.h"
#include "lxa.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <math.h>

extern FD_lxa *lxa;
extern base_info BASE;
extern channel_info CHAN;
extern horizontal_info HOR;
extern int n,chunk;
extern float *dx,*dy1,*dy2, *tbuf[2], *rbuf;
//************************************************************************
//
// this subroutine reads and displays the content of a raw file
//
int plot_file(FL_OBJECT *xyplot, int button)
{
  int ds=BASE.disp_size;
  float mdx[6]={0,0,0,0,0,ds},mdy[6]={0,0,5,-5,0,0};
  int channel[2]; 
  int chan;
  float sc;
  char cad[16];
  int xind;  // for mouse pointer
  float daty[2][5*DEFAULT_SAMPLING_RATE+1];
  channel[0]=HOR.trig_chan; 
  channel[1]=HOR.trig_chan==0?1:0;        // not trig channel
  chan=channel[0];

int shift,done;
int k;
int nbsamples=BASE.nbsamples*8;  // chunk size
int rcount; // number of samples in rbuf
 
if(button==0) {		// initial readin:
BASE.tstart=0;
BASE.tcount=0;
BASE.idatapos=0;	
fl_set_scaler_value(lxa->counter_Pos,BASE.idatapos);
int done=0;				
// tbuf[0] is 4096*64=262144 samples (float) for each channel
// rbuf is 4096*16=65536 samples (short) for both channels
// thus 8 rbuf fill (both) tbuf

while (!done) {			
	rcount=read(BASE.ihandle,rbuf,nbsamples*4);
	if(rcount==0) {
		close(BASE.ihandle);
		BASE.ifile=2;
		done=1;
	}
	else {
			for (k=0;k<rcount/4;k++)  {
			 tbuf[0][BASE.tcount+k]=get_sample(0,k);
			 tbuf[1][BASE.tcount+k]=get_sample(1,k);
			}
			BASE.tcount=BASE.tcount+rcount/4;
			if(BASE.tcount>=	nbsamples*8) done=1;	
	}
	if(rcount!=nbsamples*4) {
		close(BASE.ihandle);
		BASE.ifile=2;
		done=1;
	}
}
		if(ds>BASE.tcount) {
			ds=BASE.tcount;
		}
			// fill dx to full display size, to allow proper scaling
		for (int cont=0;cont<BASE.disp_size;cont++) {
     		dx[cont]=cont*1000.0/BASE.sampling_rate;
    	}
    	if (CHAN.on[chan]) {
        for (int cont=0;cont<ds;cont++) 
            daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    	}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    }
// other channel
    	chan=channel[1];
    	if (CHAN.on[chan]) {
        	for (int cont=0;cont<ds;cont++) 
            	daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    		}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    	}
}
else if (button==1) {
	shift=(BASE.disp_size+3)/8; // smallest disp_size is 5; 
								// need 8 for getting out 1 after division
	if(BASE.tstart+BASE.disp_size+shift<BASE.tcount) {
		BASE.tstart=BASE.tstart+shift;
		for (int cont=0;cont<BASE.disp_size;cont++) {
     		dx[cont]=cont*1000.0/BASE.sampling_rate;
    	}
    	if (CHAN.on[chan]) {
        for (int cont=0;cont<ds;cont++) 
            daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    	}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    }
// other channel
    	chan=channel[1];
    	if (CHAN.on[chan]) {
        	for (int cont=0;cont<ds;cont++) 
            	daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    		}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    	}
	}
	else {  // shift and read more data
		if(BASE.ifile==1) {
			rcount=read(BASE.ihandle,rbuf,nbsamples*4);
			if(rcount==0) {
				close(BASE.ihandle);
				BASE.ifile=2;
			}
			else {
 				for (k=0;k<7*nbsamples;k++)  {
					tbuf[0][k]=tbuf[0][nbsamples+k];
					tbuf[1][k]=tbuf[1][nbsamples+k];
				}
				for (k=0;k<rcount/4;k++)  {
				 tbuf[0][7*nbsamples+k]=get_sample(0,k);
				 tbuf[1][7*nbsamples+k]=get_sample(1,k);
				}
				BASE.tstart=BASE.tstart-nbsamples;
				BASE.tcount=BASE.tcount-nbsamples+rcount/4;
				if(BASE.tcount<0) BASE.tcount=0;
				}
				if(rcount!=nbsamples*4) {
					close(BASE.ihandle);
					BASE.ifile=2;
				}
		}
			if(BASE.tstart<0) {    
				shift=-BASE.tstart;
				BASE.tstart=0;
				for (int cont=0;cont<BASE.disp_size;cont++) {
     				dx[cont]=cont*1000.0/BASE.sampling_rate;
    			}
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
           	 			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    			}
// other channel
    			chan=channel[1];
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
            			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    			}	
			}
			else if(BASE.tstart+BASE.disp_size+shift<BASE.tcount) {
				BASE.tstart=BASE.tstart+shift;
				for (int cont=0;cont<BASE.disp_size;cont++) {
     				dx[cont]=cont*1000.0/BASE.sampling_rate;
    			}
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
           	 			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    			}
// other channel
    			chan=channel[1];
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
            			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    			}	
			}
			else { 
				ds=BASE.tcount-shift-BASE.tstart;
				if(ds<=0) {
					printf("end of data reached\n"); return 0; 
				}
				BASE.tstart=BASE.tstart+shift;
				for (int cont=0;cont<BASE.disp_size;cont++) {
     				dx[cont]=cont*1000.0/BASE.sampling_rate;
    			}
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
           	 			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    			}
// other channel
    			chan=channel[1];
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
            			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    			}		
		}
	}    
				BASE.idatapos=BASE.idatapos+shift;	
				fl_set_scaler_value(lxa->counter_Pos,1000*(BASE.idatapos/BASE.sampling_rate));		
}
else if (button==2) {
	shift=7*BASE.disp_size/8;
	if(BASE.tstart+BASE.disp_size+shift<BASE.tcount) {
		BASE.tstart=BASE.tstart+shift;
		for (int cont=0;cont<BASE.disp_size;cont++) {
     		dx[cont]=cont*1000.0/BASE.sampling_rate;
    	}
    	if (CHAN.on[chan]) {
        for (int cont=0;cont<ds;cont++) 
            daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    	}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    }
// other channel
    	chan=channel[1];
    	if (CHAN.on[chan]) {
        	for (int cont=0;cont<ds;cont++) 
            	daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    		}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    	}		
	}
	else {  // shift and read
		chunk=0;
		done=0;
		if(BASE.ifile==1) {
		while (!done) {
			rcount=read(BASE.ihandle,rbuf,nbsamples*4);
			if(rcount==0) {
				close(BASE.ihandle);
				BASE.ifile=2;
				done=1;
			}
			else {
 				for (k=0;k<7*nbsamples;k++)  {
					tbuf[0][k]=tbuf[0][nbsamples+k];
					tbuf[1][k]=tbuf[1][nbsamples+k];
				}
				for (k=0;k<rcount/4;k++)  {
				 tbuf[0][7*nbsamples+k]=get_sample(0,k);
				 tbuf[1][7*nbsamples+k]=get_sample(1,k);
				}
				BASE.tstart=BASE.tstart-nbsamples;
				BASE.tcount=BASE.tcount-nbsamples+rcount/4;
				if(BASE.tcount<0) BASE.tcount=0;
				}
				if(rcount!=nbsamples*4) {
					close(BASE.ihandle);
					BASE.ifile=2;
					done=1;
				}
				chunk=chunk+1;
				if(chunk>6) done=1;
				}
		}			
				if(BASE.tstart<0) {    
				shift=-BASE.tstart;
				BASE.tstart=0;
				for (int cont=0;cont<BASE.disp_size;cont++) {
     				dx[cont]=cont*1000.0/BASE.sampling_rate;
    			}
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
           	 			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    			}
// other channel
    			chan=channel[1];
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
            			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    			}	
			}
			else if(BASE.tstart+BASE.disp_size+shift<BASE.tcount) {
				BASE.tstart=BASE.tstart+shift;
				for (int cont=0;cont<BASE.disp_size;cont++) {
     				dx[cont]=cont*1000.0/BASE.sampling_rate;
    			}
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
           	 			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    			}
// other channel
    			chan=channel[1];
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
            			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    			}		
			}
			else { 
				ds=BASE.tcount-shift-BASE.tstart;
				if(ds<=0) {
					printf("end of data reached\n"); return 0; 
				}
				BASE.tstart=BASE.tstart+shift;
				for (int cont=0;cont<BASE.disp_size;cont++) {
     				dx[cont]=cont*1000.0/BASE.sampling_rate;
    			}
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
           	 			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    			}
// other channel
    			chan=channel[1];
    			if (CHAN.on[chan]) {
        			for (int cont=0;cont<ds;cont++) 
            			daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    			}
    			else  {
        			for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    			}		
		}
	}	
	BASE.idatapos=BASE.idatapos+shift;	
	fl_set_scaler_value(lxa->counter_Pos,1000*(BASE.idatapos/BASE.sampling_rate));	
	
}
else if(button==3) {
		if(ds>BASE.tcount-BASE.tstart) ds=BASE.tcount-BASE.tstart;
		for (int cont=0;cont<BASE.disp_size;cont++) {
     		dx[cont]=cont*1000.0/BASE.sampling_rate;
    	}
    	if (CHAN.on[chan]) {
        for (int cont=0;cont<ds;cont++) 
            daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    	}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    }
// other channel
    	chan=channel[1];
    	if (CHAN.on[chan]) {
        	for (int cont=0;cont<ds;cont++) 
            	daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    		}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    	}
}
else if (button<0) {
	if(BASE.tstart==0) {
		printf("reached earliest data available in memory\n");
		return 0;
	}
	shift=(BASE.disp_size+3)/8;
	if(button==-2) shift=7*BASE.disp_size/8;
	if(shift>BASE.tstart) shift=BASE.tstart;
	BASE.tstart=BASE.tstart-shift;
		if(ds>BASE.tcount-BASE.tstart) ds=BASE.tcount-BASE.tstart;
		for (int cont=0;cont<BASE.disp_size;cont++) {
     		dx[cont]=cont*1000.0/BASE.sampling_rate;
    	}
    	if (CHAN.on[chan]) {
        for (int cont=0;cont<ds;cont++) 
            daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    	}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;
    }
// other channel
    	chan=channel[1];
    	if (CHAN.on[chan]) {
        	for (int cont=0;cont<ds;cont++) 
            	daty[chan][cont]=tbuf[chan][cont+BASE.tstart];
    		}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont+BASE.tstart]=0;    
    	}		
    BASE.idatapos=BASE.idatapos-shift;	
	fl_set_scaler_value(lxa->counter_Pos,1000*(BASE.idatapos/BASE.sampling_rate));		
	}
else {
	printf("not implemented functionality\n");
	return 0;
}
	
// calculate and update channel information
float s;
for (int chan=0; chan<2;chan++) {
   CHAN.max[chan]=-1000000;
   CHAN.min[chan]=1000000;
   CHAN.rms[chan]=0;
   CHAN.avg[chan]=0;
   for (int cont=0; cont<ds; cont++) {
      s=daty[chan][cont];
      if (s<CHAN.min[chan]) CHAN.min[chan]=s;
      if (s>CHAN.max[chan]) CHAN.max[chan]=s;
      CHAN.rms[chan]=CHAN.rms[chan]+s*s;
	  CHAN.avg[chan]=CHAN.avg[chan]+s;
    }
	 if(CHAN.rms[chan]>0) CHAN.rms[chan]=sqrt(CHAN.rms[chan]/ds);
	 else CHAN.rms[chan]=0;
     CHAN.pp[chan]=CHAN.max[chan]-CHAN.min[chan];
     CHAN.avg[chan]=CHAN.avg[chan]/ds;
	}


  // *********** Channel 1 ******************
  sprintf(cad,"%.3f",CHAN.min[0]);
  fl_set_object_label(lxa->text_min_chan1,cad);

  sprintf(cad,"%.3f",CHAN.max[0]);
  fl_set_object_label(lxa->text_max_chan1,cad);

  sprintf(cad,"%.3f",CHAN.pp[0]);
  fl_set_object_label(lxa->text_pp_chan1,cad);

  sprintf(cad,"%.3f",CHAN.avg[0]);
  fl_set_object_label(lxa->text_avg_chan1,cad);

  sprintf(cad,"%.3f",CHAN.rms[0]);
  fl_set_object_label(lxa->text_avg_chan1,cad);

  // *********** Channel 2 ******************
  sprintf(cad,"%.3f",CHAN.min[1]);
  fl_set_object_label(lxa->text_min_chan2,cad);

  sprintf(cad,"%.3f",CHAN.max[1]);
  fl_set_object_label(lxa->text_max_chan2,cad);

  sprintf(cad,"%.3f",CHAN.pp[1]);
  fl_set_object_label(lxa->text_pp_chan2,cad);

  sprintf(cad,"%.3f",CHAN.avg[1]);
  fl_set_object_label(lxa->text_avg_chan2,cad);

  sprintf(cad,"%.3f",CHAN.rms[1]);
  fl_set_object_label(lxa->text_avg_chan2,cad);

	xind=0;
	while (BASE.pointer_update) {
	   if(dx[xind]>=BASE.wx) {
			sprintf(cad,"%.2f",daty[0][xind]);
  			fl_set_object_label(lxa->text_mouse_V1,cad);
			sprintf(cad,"%.2f",daty[1][xind]);
  			fl_set_object_label(lxa->text_mouse_V2,cad);
			BASE.pointer_update=0;
		}
		xind++;
		if(xind>ds) BASE.pointer_update=0;
	}



switch (BASE.op_mode) {
   case OPERATION_MODE_NORMAL: {
       mdx[0]=dx[0];
	   mdx[5]=dx[BASE.disp_size-1];
	   fl_set_xyplot_xbounds(xyplot,dx[0],dx[BASE.disp_size-1]);
       fl_set_xyplot_data(xyplot,mdx,mdy,6,"","time(ms)","");
//     want yellow lines for x=0 and y=0:
       fl_add_xyplot_overlay(xyplot,1,mdx,mdy,6,FL_YELLOW);
	   for (int ch=0; ch<2;ch++) {
			chan=channel[ch];	   
			sc=1.0/CHAN.V_scale[chan];
            if (CHAN.inv[chan]==1) sc*=-1;
            for (int cont=0; cont<ds; cont++) {
	           daty[chan][cont]=CHAN.V_pos[chan]+sc*daty[chan][cont];
	   	    }
       		if (CHAN.on[chan]) {
           		fl_add_xyplot_overlay(xyplot,chan+2,dx,daty[chan],ds,CHAN.color[chan]);
           	 }
       		else fl_delete_xyplot_overlay(xyplot,chan+2);
		}
       break;
     }
     case OPERATION_MODE_ADD: {
         mdx[0]=dx[0];
         mdx[5]=dx[BASE.disp_size-1];
         fl_set_xyplot_xbounds(xyplot,dx[0],dx[BASE.disp_size-1]);
         fl_set_xyplot_data(xyplot,mdx,mdy,6,"","time(ms)","");
//     want yellow lines for x=0 and y=0:
         fl_add_xyplot_overlay(xyplot,1,mdx,mdy,6,FL_YELLOW);
		 float sc0=1.0/CHAN.V_scale[0];
		 float sc1=1.0/CHAN.V_scale[0];
		 if (CHAN.inv[0]==1) sc0*=-1;
		 if (CHAN.inv[1]==1) sc1*=-1;
         for (int cont=0;cont<ds;cont++) {
             daty[0][cont]=CHAN.V_pos[0]+sc0*daty[0][cont]+sc1*daty[1][cont];
         }
         fl_add_xyplot_overlay(xyplot,2,dx,daty[0],ds,CHAN.color[0]);
         fl_delete_xyplot_overlay(xyplot,3);
         break;
     }
     case OPERATION_MODE_XY: {
         chan=channel[0];
         mdx[0]=-5.0;
         mdx[5]=5.0;
         sprintf(cad,"Channel %d --->" ,channel[0]+1);
         fl_set_xyplot_xbounds(xyplot,-5.0,5.0);
	     fl_set_xyplot_alphaxtics(xyplot,
			"@-5.0|@-4.0|@-3.0|-@-2.0|@-1.0|0@0.0|@1.0|+@2.0|@3.0|@4.0|@5.0","");
         fl_set_xyplot_data(xyplot,mdx,mdy,6,"",cad,"");
// want yellow lines for x=0 and y=0:
         fl_add_xyplot_overlay(xyplot,1,mdx,mdy,6,FL_YELLOW); 	   
		 for (int ch=0; ch<2;ch++) {
			chan=channel[ch];	   
			sc=1.0/CHAN.V_scale[chan];
            if (CHAN.inv[chan]==1) sc*=-1;
            for (int cont=0; cont<ds; cont++) {
	           daty[chan][cont]=CHAN.V_pos[chan]+sc*daty[chan][cont];
	   	    }
		 }      
         fl_add_xyplot_overlay(xyplot,2,daty[channel[0]],daty[channel[1]],ds,CHAN.color[channel[0]]);
         fl_delete_xyplot_overlay(xyplot,3);
         break;
     }
}   // switch
  return 1;
}



